; Title:  "NewKidOnTheBlock"
; Chip:   6581 C64 used when composing.
; Author: Frantic/Hack'n'Trade

;---------------------------------
; CONSTANTS
;

NUMBEROFVOICES    = 2 ;Using all three Voices
S                 = $0e ;Song "speed"

;---------------------------------
; ZERO PAGE ADDRESSES
;

TICKCOUNTERS      = $02 ;Use zp for tickcounters.. TICKCOUNTERS+0, TICKCOUNTERS+7,
        ;TICKCOUNTERS+14 will be used.
DATAPOS           = $03 ;Use zp for SONGPOSITIONS... 0, 7, 14.. 
FILTERHI          = $23


init_sid:    
;    sei

    ldy #$02  ;Set it to $02 so the first incorrectly timed 
        ;iteration won't affect anything
    sty FILTERHI  ;Init filtersweep to make it a bit more deterministic.

    ;Init counters and sonpositions
    lda #(NUMBEROFVOICES-1)*7 ;Voice indeX

initlp:  
    tax
    ;sta DATAPOS,x   ;Init datapos+0 to 0, datapos+7 to 7 and datapos+14 to 14.
    sty TICKCOUNTERS,x  ;y can be anything.. doesn't matter, but now I happen to 
          ;init the filter so set it to $01 too
    sec
    sbc #7
    bpl initlp

    lda #<(Voc0start-musicdata)
    sta  DATAPOS

    lda #<(Voc1start-musicdata)
    sta  DATAPOS + 7
    
    rts

play_sid:
    jsr $b5ff ;hijack some BASIC ROM shit to do the filter sweep
          ;b5ff looks like this:
          ; inc $23
          ; ldx $23
          ; ldy #$00
          ; rts
    stx $d416 ;filter hi

    ldx #(NUMBEROFVOICES-1)*7 ;Voice indeX
    ;Right here is a good place for Voice specific code, if there is any reason for that.
innerloop:
    dec TICKCOUNTERS,x
    bne loopend
    
    ;Time for new sound settings, turn gate and oscillator off..
    lda #8
    sta $d404,x

    ;Turn on global volume to make sure we'll hear anything at all.
    ;Reason for having this code *inside* the loop: 
    ;  To make sure the player won't run too fast and get executed
    ;  twice on the same rasterline.
    lda #$5f  ;Hi-pass + Lo-pass filter on.
    sta $d418
    ;lda #$a2  ;Filter used on middle (melody) voice.
    sta $d417
    
    ;Parse data sequence data
    ldy DATAPOS,x
  newseq:
    iny
    lda musicdata-1,y  ;Get databyte
    bne nonewseq
    lda musicdata-0,y  ;Get jumpval if it's jumptime
    tay     ;..and use new one instead
    bpl newseq   ;This means data may not be larger than $80 bytes
  nonewseq:
    sty DATAPOS,x
    pha
    and #$0f    ;Note value
    tay
    lda freqhi,y
    sta $d401,x   ;Freq hi
    lda freqlo,y
    sta $d400,x   ;Freq lo
    pla
    lsr
    lsr
    lsr
    lsr
    tay
    lda adtab,y
    sta $d405,x
    lda durtab,y
    sta TICKCOUNTERS,x
    lda ctrltab,y
    sta $d404,x
    
loopend:
    txa
    !byte $cb,7   ;axs #7 / sbx #7 / whatever..
    bpl innerloop
    
    rts
    ;bmi outerloop

    

;---------------------------------
; "Instruments"
;
; Using AD only to save space. (SR is set to 00 as default)
;
; A pattern is S*8 ticks long, so using "instrument" 5 we
; can represent a whole empty pattern by just one byte in
; the sequence data. At the same time, this format allows
; for changes to the waveform every tick, which means we
; can also make drums and such things. But, not in this 
; tune. Perhaps in the next one..
;
;               00   01   02   03   04   05   06  
adtab:   !byte $ad, $ad, $ad, $ad, $ad, $00, $ad
ctrltab: !byte $21, $21, $21, $21, $21, $00, $21
durtab:  !byte S*1, S*2, S*3, S*4, S*5, S*6, S*6

;---------------------------------
;Note freq data
;
; Only using needed notes.

    JP  =  0
    
    B2  =  1
    C3  =  2
    Cz3 =  3
    D3  =  4
    Dz3 =  5
    E3  =  6
    F3  =  7
    Fz3 =  8
    G3  =  9
    Gz3 =  10
    A3  =  11
    Az3 =  12
    B3  =  13
    ;D4  =  14
    ;E4  =  15

freqlo = *-1 ;-1 to exclude the corresponding first byte (in freqhi) used as a "wrap flag"
    ;      C   Cz  D   Dz  E   F   Fz  G   Gz  A   Az  B
    !byte                                             $1c  ; 2
    !byte $5a,$9c,$e2,$2d,$7c,$cf,$28,$85,$e8,$52,$c1,$37  ; 3


freqhi:
   !byte $00 ;WRAP
  ;      C   Cz  D   Dz  E   F   Fz  G   Gz  A   Az  B
  !byte                                             $04  ; 2
  !byte $04,$04,$04,$05,$05,$05,$06,$06,$06,$07,$07,$08  ; 3
    
;;https://codebase64.org/doku.php?id=base:pal_frequency_table    
;FreqTablePalLo:
;;      C   C#  D   D#  E   F   F#  G   G#  A   A#  B
;.byte $17,$27,$39,$4b,$5f,$74,$8a,$a1,$ba,$d4,$f0,$0e  ; 1
;.byte $2d,$4e,$71,$96,$be,$e8,$14,$43,$74,$a9,$e1,$1c  ; 2
;.byte $5a,$9c,$e2,$2d,$7c,$cf,$28,$85,$e8,$52,$c1,$37  ; 3
;.byte $b4,$39,$c5,$5a,$f7,$9e,$4f,$0a,$d1,$a3,$82,$6e  ; 4
;.byte $68,$71,$8a,$b3,$ee,$3c,$9e,$15,$a2,$46,$04,$dc  ; 5
;.byte $d0,$e2,$14,$67,$dd,$79,$3c,$29,$44,$8d,$08,$b8  ; 6
;.byte $a1,$c5,$28,$cd,$ba,$f1,$78,$53,$87,$1a,$10,$71  ; 7
;.byte $42,$89,$4f,$9b,$74,$e2,$f0,$a6,$0e,$33,$20,$ff  ; 8
;
;FreqTablePalHi:
;;      C   C#  D   D#  E   F   F#  G   G#  A   A#  B
;.byte $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$02  ; 1
;.byte $02,$02,$02,$02,$02,$02,$03,$03,$03,$03,$03,$04  ; 2
;.byte $04,$04,$04,$05,$05,$05,$06,$06,$06,$07,$07,$08  ; 3
;.byte $08,$09,$09,$0a,$0a,$0b,$0c,$0d,$0d,$0e,$0f,$10  ; 4
;.byte $11,$12,$13,$14,$15,$17,$18,$1a,$1b,$1d,$1f,$20  ; 5
;.byte $22,$24,$27,$29,$2b,$2e,$31,$34,$37,$3a,$3e,$41  ; 6
;.byte $45,$49,$4e,$52,$57,$5c,$62,$68,$6e,$75,$7c,$83  ; 7
;.byte $8b,$93,$9c,$a5,$af,$b9,$c4,$d0,$dd,$ea,$f8,$ff  ; 8    

;---------------------------------
; Sequence data
;
;
; Note and duration stored in one byte and the byte following a $00 (JP)
; byte is interpreted as the destination of a jump to another place in 
; the data.

musicdata:
Voc0start:
  !byte $20 | E3
  !byte $00 | E3
  !byte $00 | G3
  !byte $00 | E3
  !byte $00 | D3
  !byte $40 | C3
  !byte $60 | B2 ; 18 Ticks
  

  !byte $20 | E3
  !byte $00 | E3
  !byte $00 | G3
  !byte $00 | E3
  !byte $00 | D3
  !byte $40 | C3
  !byte $60 | B2

  !byte $20 | E3
  !byte $00 | E3
  !byte $00 | G3
  !byte $00 | E3
  !byte $00 | D3
  !byte $40 | C3
  !byte $60 | B2
  
  !byte $00 | D3
  !byte $00 | D3
  !byte $00 | D3
  !byte $00 | D3
  !byte $00 | D3
  !byte $00 | D3
  !byte $00 | D3
  !byte $00 | D3
  
  !byte $00 | A3
  !byte $00 | A3
  !byte $00 | A3
  !byte $00 | A3
  !byte $00 | A3
  !byte $00 | A3
  !byte $00 | A3
  !byte $00 | A3  

  !byte $20 | E3
  !byte $00 | E3
  !byte $00 | G3
  !byte $00 | E3
  !byte $00 | D3
  
  !byte $60 | C3
  !byte $60 | B2
  
  !byte $20 | E3
  !byte $00 | E3
  !byte $00 | G3
  !byte $00 | E3
  !byte $00 | D3
  
  !byte $60 | C3
  !byte $60 | B2  
  
  !byte $00 | D3
  !byte $00 | D3
  !byte $00 | D3
  !byte $00 | D3
  !byte $00 | D3
  !byte $00 | D3
  !byte $00 | D3
  !byte $00 | D3
  
  !byte $00 | A3
  !byte $00 | A3
  !byte $00 | A3
  !byte $00 | A3
  !byte $00 | A3
  !byte $00 | A3
  !byte $00 | A3
  !byte $00 | A3 
  
;  !byte $20 | B3 
;;
;  !byte $00 | E3 
;  !byte $00 | G3 
;  !byte $00 | E3 
;  !byte $00 | D3 
;  !byte $20 | C3 
;  !byte $20 | B3 
  
  !byte JP
  !byte <(Voc0start-musicdata)

Voc1start:
  
  !byte $50 | 0
  !byte $50 | 0  
  !byte $50 | 0
  
  !byte $50 | 0
  !byte $50 | 0
  !byte $50 | 0 

  !byte $50 | 0
  !byte $50 | 0
  !byte $50 | 0   
  
  !byte $00 | G3
  !byte $00 | G3
  !byte $00 | G3
  !byte $00 | G3
  !byte $00 | G3
  !byte $00 | G3
  !byte $00 | G3
  !byte $00 | G3

  !byte $00 | E3
  !byte $00 | E3
  !byte $00 | E3
  !byte $00 | E3
  !byte $00 | E3
  !byte $00 | E3
  !byte $00 | E3
  !byte $00 | E3  
  
  !byte $20 | B3
  !byte $00 | B3
  !byte $00 | D3
  !byte $00 | B3
  !byte $00 | A3 

  !byte $60 | G3
  !byte $60 | Fz3
  
  !byte $20 | B3
  !byte $00 | B3
  !byte $00 | D3
  !byte $00 | B3
  !byte $00 | A3 

  !byte $60 | G3
  !byte $60 | Fz3  

  !byte $00 | G3
  !byte $00 | G3
  !byte $00 | G3
  !byte $00 | G3
  !byte $00 | G3
  !byte $00 | G3
  !byte $00 | G3
  !byte $00 | G3

  !byte $00 | E3
  !byte $00 | E3
  !byte $00 | E3
  !byte $00 | E3
  !byte $00 | E3
  !byte $00 | E3
  !byte $00 | E3
  !byte $00 | E3 
  
;  !byte $20 | E3 
  ;
  ;!byte $50 | 0
  ;!byte $50 | 0 
  
  !byte JP
  !byte <(Voc1start-musicdata)
  

